package org.fluidproject.vulab.rascal.core;


import java.awt.AWTException;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.PointerInfo;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;
import java.util.Iterator;
import java.util.logging.Logger;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;

import org.fluidproject.vulab.rascal.core.utils.FrameBuffer;
import org.fluidproject.vulab.rascal.ui.ScreencastApplet;
import org.fluidproject.vulab.rascal.utils.http.HTTPStreamer;


public class ScreenCapture {
	
	public static Logger logger = Logger.getLogger(ScreencastApplet.class.getName());


	// private Vector<BufferedImage> frames;
	FrameBuffer buffer;
	//FTPBuffer ftp;
	private Robot robot = null;
	private Thread videoThread;

	public ScreenCapture() {
		buffer = new FrameBuffer();
		
		try {
			robot = new Robot();
		} catch (AWTException e) {
			robot = null;
			e.printStackTrace();
		}
	}

	public void start() {
		captureVideo();
	}

	public void stop() {
		videoThread.interrupt();
		buffer.finalize();
	}

	

	public static BufferedImage compress(BufferedImage image, float quality) {
		try {
			Iterator<ImageWriter> writers = ImageIO
					.getImageWritersBySuffix("jpeg");
			ImageWriter writer = writers.next();
			ImageWriteParam param = writer.getDefaultWriteParam();
			param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
			param.setCompressionQuality(quality);
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			writer.setOutput(ImageIO.createImageOutputStream(out));
			writer.write(null, new IIOImage(image, null, null), param);
			byte[] data = out.toByteArray();
			ByteArrayInputStream in = new ByteArrayInputStream(data);
			
			out.close();
			BufferedImage result = ImageIO.read(in);
			in.close();
			writer = null;
			writers = null;
			param = null;
			return result;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public static BufferedImage scale(BufferedImage image, float scale) {
		int width = (int)(image.getWidth()*scale);
		int height = (int)(image.getHeight()*scale);
		Image img = image.getScaledInstance(width,height, Image.SCALE_SMOOTH);
		image.getGraphics().dispose();
		BufferedImage result = new BufferedImage(width, height,BufferedImage.TYPE_INT_RGB);
		result.createGraphics().drawImage(img, 0, 0, null);
		img = null;
		image = null;
		return result;
	
	}



	private void captureVideo() {
		try {

			videoThread = new Thread(new CaptureVideoThread());
			videoThread.start();

		} catch (Exception e) {
			e.printStackTrace();

		}// end catch
	}// end captureAudio method

	private class CaptureVideoThread extends Thread {
		public void run() {

			try {
				//HTTPStreamer streamer = null;
				Rectangle screenSize = new Rectangle(Toolkit
						.getDefaultToolkit().getScreenSize());
				while (true) {
					
					BufferedImage image = robot
							.createScreenCapture(screenSize);
					
					PointerInfo pInfo = MouseInfo.getPointerInfo();
					Point point = pInfo.getLocation();
					image = drawCursor(image, point);

					image = compress(image, 0.5f);
					image = scale(image, 0.5f);
					logger.info("JVM Memory: "+Runtime.getRuntime().maxMemory());
					logger.info("Free Memory:"+Runtime.getRuntime().freeMemory());
					
					Date now = new Date();
					long index = now.getTime();
					buffer.addFrame(index + ".jpg", image);
					if (buffer.getSize() > 10){
						buffer.flushtoFile();
					}
					image = null;
					synchronized (this) {
						System.gc();
						sleep(100); // 10 fps
					}


				}
			} catch (InterruptedException e) {
				// This means the data collection is done
				// saveFrames();
			} catch (HeadlessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} 

		}

	}

	public static BufferedImage drawCursor(BufferedImage image, Point p) {
		Graphics2D g = image.createGraphics();
		g.setBackground(Color.RED);

		g.clearRect((int) p.getX(), (int) p.getY(), 10, 10);

		g.dispose();

		return image;
	}

}